

# CNT01 - QUAD QUADRATURE ENCODER 1.2.0


# Imports
import collections
from ctypes import *
from . import Device
from . import Error


# Class CNT01
class CNT01(Device):

    # Channels
    CHANNELS = 4

    # Modes
    MODE_NORMAL = 0
    MODE_QUADRATURE_1X = 1
    MODE_QUADRATURE_2X = 2
    MODE_QUADRATURE_4X = 3

    # Configure Interface
    Device._lib.cnt01_clear.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte]
    Device._lib.cnt01_clear.restype = c_ubyte
    Device._lib.cnt01_disable.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte]
    Device._lib.cnt01_disable.restype = c_ubyte
    Device._lib.cnt01_enable.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte]
    Device._lib.cnt01_enable.restype = c_ubyte
    Device._lib.cnt01_read.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_longlong), POINTER(c_ubyte),
        POINTER(c_ulonglong)]
    Device._lib.cnt01_read.restype = c_ubyte
    Device._lib.cnt01_set_mode.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_ubyte), c_ubyte]
    Device._lib.cnt01_set_mode.restype = c_ubyte

    # Named Tuples
    _Counter = collections.namedtuple('Counter', 'value time')


    # Clear Counter
    def clear(self, *args):

        # Check Parameters
        if len(args) == 0:

            # Set Channel List
            channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))

        elif len(args) == 1:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List
                channel = self._array(c_ubyte, [args[0]])

            elif isinstance(args[0], set):

                # Set Channel List
                channel = self._array(c_ubyte, args[0])

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Clear Counter
        error = self._lib.cnt01_clear(self._address, channel, self._value(c_ubyte, len(channel)))

        # Error Handler
        self._error(error)


    # Disable Counter
    def disable(self, *args):

        # Check Parameters
        if len(args) == 0:

            # Set Channel List
            channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))

        elif len(args) == 1:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List
                channel = self._array(c_ubyte, [args[0]])

            elif isinstance(args[0], set):

                # Set Channel List
                channel = self._array(c_ubyte, args[0])

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Disable Counter
        error = self._lib.cnt01_disable(self._address, channel, self._value(c_ubyte, len(channel)))

        # Error Handler
        self._error(error)


    # Enable Counter
    def enable(self, *args):

        # Check Parameters
        if len(args) == 0:

            # Set Channel List
            channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))

        elif len(args) == 1:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List
                channel = self._array(c_ubyte, [args[0]])

            elif isinstance(args[0], set):

                # Set Channel List
                channel = self._array(c_ubyte, args[0])

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Enable Counter
        error = self._lib.cnt01_enable(self._address, channel, self._value(c_ubyte, len(channel)))

        # Error Handler
        self._error(error)


    # Set Mode
    def mode(self, *args):

        # Check Parameters
        if len(args) == 1:

            # Check Mode
            if isinstance(args[0], int):

                # Set default Channel List and Mode List
                channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))
                mode = self._array(c_ubyte, [args[0]] * self.CHANNELS);

            elif isinstance(args[0], dict):

                # Set Channel List and Mode List
                channel = self._array(c_ubyte, args[0].keys())
                mode = self._array(c_ubyte, args[0].values())

            else:

                # Error
                raise Error.Parameter

        elif len(args) == 2:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List and Mode List
                channel = self._array(c_ubyte, [args[0]])
                mode = self._array(c_ubyte, [args[1]])

            elif isinstance(args[0], set):

                # Set Channel List and Mode List
                channel = self._array(c_ubyte, args[0])
                mode = self._array(c_ubyte, [args[1]] * len(args[0]))

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Set Mode
        error = self._lib.cnt01_set_mode(self._address, channel, self._value(c_ubyte, len(channel)), mode,
            self._value(c_ubyte, len(mode)))

        # Error Handler
        self._error(error)


    # Read Counter
    def read(self, *args):

        # Check Parameters
        if len(args) == 0:

            # Set default Channel List
            channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))

        elif len(args) == 1:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List
                channel = self._array(c_ubyte, [args[0]])

            elif isinstance(args[0], set):

                # Set Channel List
                channel = self._array(c_ubyte, args[0])

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Value List and Time
        value = self._array(c_longlong, len(channel))
        time = self._value(c_ulonglong)

        # Read Counter
        error = self._lib.cnt01_read(self._address, channel, self._value(c_ubyte, len(channel)), value,
            self._value(c_ubyte, len(value)), time)

        # Error Handler
        self._error(error)

        # Value List
        value_ = {}

        # Parse Channel List
        for i in range(len(channel)):

            # Insert Value into List
            value_[channel[i]] = value[i]

        # Return Counter
        return self._Counter(value_ if ((len(args) == 0) or isinstance(args[0], set)) else value_[channel[0]],
            time.value)
