

# DO01 - OCTAL DIGITAL OUTPUT BUFFER 1.1.0


# Imports
from ctypes import *
from . import Device
from . import Error


# Class DO01
class DO01(Device):

    # Channels
    CHANNELS = 8

    # States
    STATE_LOW = False
    STATE_HIGH = True

    # Configure Library
    Device._lib.do01_measure.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_float), POINTER(c_ubyte)]
    Device._lib.do01_measure.restype = c_ubyte
    Device._lib.do01_write.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_ubyte), c_ubyte]
    Device._lib.do01_write.restype = c_ubyte


    # Measure Current [A]
    def measure(self, *args):

        # Check Parameters
        if len(args) == 0:

            # Set default Channel List
            channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))

        elif len(args) == 1:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List
                channel = self._array(c_ubyte, [args[0]])

            elif isinstance(args[0], set):

                # Set Channel List
                channel = self._array(c_ubyte, args[0])

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Set Current List
        current = self._array(c_float, len(channel))

        # Measure Current
        error = self._lib.do01_measure(self._address, channel, self._value(c_ubyte, len(channel)), current,
            self._value(c_ubyte, len(current)))

        # Error Handler
        self._error(error)

        # Current List
        current_ = {}

        # Parse Channel List
        for i in range(len(channel)):

            # Insert Current into List
            current_[channel[i]] = current[i]

        # Return Current List
        return current_ if ((len(args) == 0) or isinstance(args[0], set)) else current_[channel[0]]


    # Write
    def write(self, *args):

        # Check Parameters
        if len(args) == 1:

            # Check State
            if isinstance(args[0], dict):

                # Set Channel List and State List
                channel = self._array(c_ubyte, args[0].keys())
                state = self._array(c_ubyte, args[0].values())

            else:

                # Set default Channel List and State List
                channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))
                state = self._array(c_ubyte, [args[0]] * self.CHANNELS);

        elif len(args) == 2:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel and State List
                channel = self._array(c_ubyte, [args[0]])
                state = self._array(c_ubyte, [args[1]])

            elif isinstance(args[0], set):

                # Set Channel and State List
                channel = self._array(c_ubyte, args[0])
                state = self._array(c_ubyte, [args[1]] * len(args[0]))

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Write
        error = self._lib.do01_write(self._address, channel, self._value(c_ubyte, len(channel)), state,
            self._value(c_ubyte, len(state)))

        # Error Handler
        self._error(error)
