

# DO02 - OCTAL RELAY MODULE 1.1.0


# Imports
from ctypes import *
from . import Device
from . import Error


# Class DO02
class DO02(Device):

    # Channels
    CHANNELS = 8

    # States
    STATE_OPEN = False
    STATE_CLOSED = True

    # Configure Library
    Device._lib.do02_write.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_ubyte), c_ubyte]
    Device._lib.do02_write.restype = c_ubyte


    # Write
    def write(self, *args):

        # Check Parameters
        if len(args) == 1:

            # Check State
            if isinstance(args[0], dict):

                # Set Channel List and State List
                channel = self._array(c_ubyte, args[0].keys())
                state = self._array(c_ubyte, args[0].values())

            else:

                # Set default Channel List and State List
                channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))
                state = self._array(c_ubyte, [args[0]] * self.CHANNELS);

        elif len(args) == 2:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel and State List
                channel = self._array(c_ubyte, [args[0]])
                state = self._array(c_ubyte, [args[1]])

            elif isinstance(args[0], set):

                # Set Channel and State List
                channel = self._array(c_ubyte, args[0])
                state = self._array(c_ubyte, [args[1]] * len(args[0]))

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Write
        error = self._lib.do02_write(self._address, channel, self._value(c_ubyte, len(channel)), state,
            self._value(c_ubyte, len(state)))

        # Error Handler
        self._error(error)
