

# TMP01 - OCTAL THERMOCOUPLE MONITOR 1.2.0


# Imports
from ctypes import *
from . import Device
from . import Error


# Class TMP01
class TMP01(Device):

    # Class Sensor
    class Sensor:

        # Types
        TYPE_NONE = 0
        TYPE_CUSTOM = 1
        TYPE_B = 2
        TYPE_C = 3
        TYPE_D = 4
        TYPE_E = 5
        TYPE_G = 6
        TYPE_J = 7
        TYPE_K = 8
        TYPE_M = 9
        TYPE_N = 10
        TYPE_P = 11
        TYPE_R = 12
        TYPE_S = 13
        TYPE_T = 14


        # Constructor
        def __init__(self, *args):

            # Check Parameters
            if len(args) == 0:

                # Initialize
                self.type(self.TYPE_NONE)

            elif len(args) == 1:

                # Initialize
                self.type(args[0])

            else:

                # Error
                raise Error.Parameter


        # Type
        def type(self, *args):

            # Check Parameters
            if len(args) == 0:

                # Return Type
                return self._type

            elif len(args) == 1:

                # Check Type
                if ((args[0] != self.TYPE_NONE) and (args[0] != self.TYPE_CUSTOM) and (args[0] != self.TYPE_B) and
                    (args[0] != self.TYPE_C) and (args[0] != self.TYPE_D) and (args[0] != self.TYPE_E) and
                    (args[0] != self.TYPE_G) and (args[0] != self.TYPE_J) and (args[0] != self.TYPE_K) and
                    (args[0] != self.TYPE_M) and (args[0] != self.TYPE_N) and (args[0] != self.TYPE_P) and
                    (args[0] != self.TYPE_R) and (args[0] != self.TYPE_S) and (args[0] != self.TYPE_T)):

                    # Error
                    raise Error.Parameter

                # Set Type
                self._type = args[0]

            else:

                # Error
                raise Error.Parameter


    # Channels
    CHANNELS = 8

    # Sampling Frequencies [Hz]
    FREQUENCY_6 = 6
    FREQUENCY_12 = 12
    FREQUENCY_25 = 25
    FREQUENCY_50 = 50
    FREQUENCY_100 = 100
    FREQUENCY_200 = 200
    FREQUENCY_400 = 400
    FREQUENCY_800 = 800
    FREQUENCY_1500 = 1500
    FREQUENCY_3000 = 3000

    # Configure Library
    Device._lib.tmp01_measure.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_float), POINTER(c_ubyte)]
    Device._lib.tmp01_measure.restype = c_ubyte
    Device._lib.tmp01_set_frequency.argtypes = [c_uint, c_ushort]
    Device._lib.tmp01_set_frequency.restype = c_ubyte
    Device._lib.tmp01_set_sensor.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_ubyte), c_ubyte]
    Device._lib.tmp01_set_sensor.restype = c_ubyte


    # Measure [K]
    def measure(self, *args):

        # Check Parameters
        if len(args) == 0:

            # Set default Channel List
            channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))

        elif len(args) == 1:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List
                channel = self._array(c_ubyte, [args[0]])

            elif isinstance(args[0], set):

                # Set Channel List
                channel = self._array(c_ubyte, args[0])

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Temperature List
        temperature = self._array(c_float, len(channel))

        # Measure Temperature
        error = self._lib.tmp01_measure(self._address, channel, self._value(c_ubyte, len(channel)), temperature,
            self._value(c_ubyte, len(temperature)))

        # Error Handler
        self._error(error)

        # Temperature List
        temperature_ = {}

        # Parse Channel List
        for i in range(len(channel)):

            # Insert Temperature into List
            temperature_[channel[i]] = temperature[i]

        # Return Temperature List
        return temperature_ if ((len(args) == 0) or isinstance(args[0], set)) else temperature_[channel[0]]


    # Set Sensor
    def sensor(self, *args):

        # Check Parameters
        if len(args) == 1:

            # Check Sensor
            if isinstance(args[0], self.Sensor):

                # Set default Channel List and Sensor Type List
                channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))
                type = self._array(c_ubyte, [args[0].type()] * self.CHANNELS);

            elif isinstance(args[0], dict):

                # Parse Values
                for sensor in args[0].values():

                    # Check Sensor
                    if not isinstance(sensor, self.Sensor):

                        # Error
                        raise Error.Parameter

                # Set Channel List and Sensor Type List
                channel = self._array(c_ubyte, args[0].keys())
                type = self._array(c_ubyte, [sensor.type() for sensor in args[0].values()])

            else:

                # Error
                raise Error.Parameter

        elif (len(args) == 2) and isinstance(args[1], self.Sensor):

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel and Sensor Type List
                channel = self._array(c_ubyte, [args[0]])
                type = self._array(c_ubyte, [args[1].type()])

            elif isinstance(args[0], set):

                # Set Channel and Sensor Type List
                channel = self._array(c_ubyte, args[0])
                type = self._array(c_ubyte, [args[1].type()] * len(args[0]))

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Set Sensor
        error = self._lib.tmp01_set_sensor(self._address, channel, self._value(c_ubyte, len(channel)), type,
            self._value(c_ubyte, len(type)))

        # Error Handler
        self._error(error)


    # Set Frequency [Hz]
    def frequency(self, frequency):

        # Set Frequency
        error = self._lib.tmp01_set_frequency(self._address, self._value(c_ushort, frequency))

        # Error Handler
        self._error(error)
