

# VO01 - OCTAL VOLTAGE OUTPUT DEVICE 1.1.0


# Imports
from ctypes import *
from . import Device
from . import Error


# Class VO01
class VO01(Device):

    # Channels
    CHANNELS = 8

    # Configure Library
    Device._lib.vo01_control.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_float), c_ubyte]
    Device._lib.vo01_control.restype = c_ubyte


    # Control [V]
    def control(self, *args):

        # Check Parameters
        if len(args) == 1:

            # Check Voltage
            if isinstance(args[0], dict):

                # Set Channel List and Voltage List
                channel = self._array(c_ubyte, args[0].keys())
                voltage = self._array(c_float, args[0].values())

            else:

                # Set Channel List and Voltage List
                channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))
                voltage = self._array(c_float, [args[0]] * self.CHANNELS);

        elif len(args) == 2:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel and Voltage List
                channel = self._array(c_ubyte, [args[0]])
                voltage = self._array(c_float, [args[1]])

            elif isinstance(args[0], set):

                # Set Channel and Voltage List
                channel = self._array(c_ubyte, args[0])
                voltage = self._array(c_float, [args[1]] * len(args[0]))

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Control Voltage
        error = self._lib.vo01_control(self._address, channel, self._value(c_ubyte, len(channel)), voltage,
            self._value(c_ubyte, len(voltage)))

        # Error Handler
        self._error(error)
